/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.flood_duration;

import java.util.HashSet;
import java.util.Set;

import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.common.AbstractCalculationResult;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.artifacts.sinfo.flood_duration.FloodDurationCalculationResult.Infrastructure;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.DurationCurveGenerator;
import org.dive4elements.river.exports.process.DefaultProcessor;
import org.dive4elements.river.model.Attribute.AttributeKey;

/**
 * Processor to generate the facet and data series of a flood duration curve
 *
 * @author Matthias Schäfer
 *
 */
public final class FloodDurationCurveProcessor extends DefaultProcessor {

    private static final String FACET_FLOOD_DURATION_W = FacetTypes.DURATION_W; // "duration_curve.w";

    private static final String FACET_FLOOD_DURATION_Q = FacetTypes.DURATION_Q; // "duration_curve.q";

    public static final String FACET_FLOOD_DURATION_MAINVALUES_W = "sinfo.mainvalues.w";

    public static final String FACET_FLOOD_DURATION_MAINVALUES_Q = "sinfo.mainvalues.q";

    public static final String FACET_FLOOD_DURATION_INFRASTRUCTURE_W_LEFT = "infrastructure.w.left";

    public static final String FACET_FLOOD_DURATION_INFRASTRUCTURE_W_RIGHT = "infrastructure.w.right";

    public static final String FACET_FLOOD_DURATION_INFRASTRUCTURE_Q_LEFT = "infrastructure.q.left";

    public static final String FACET_FLOOD_DURATION_INFRASTRUCTURE_Q_RIGHT = "infrastructure.q.right";

    private static final Set<String> HANDLED_FACET_TYPES = new HashSet<>();

    static {
        HANDLED_FACET_TYPES.add(FACET_FLOOD_DURATION_W);
        HANDLED_FACET_TYPES.add(FACET_FLOOD_DURATION_Q);
        HANDLED_FACET_TYPES.add(FACET_FLOOD_DURATION_MAINVALUES_W);
        HANDLED_FACET_TYPES.add(FACET_FLOOD_DURATION_MAINVALUES_Q);
        HANDLED_FACET_TYPES.add(FACET_FLOOD_DURATION_INFRASTRUCTURE_W_LEFT);
        HANDLED_FACET_TYPES.add(FACET_FLOOD_DURATION_INFRASTRUCTURE_W_RIGHT);
        HANDLED_FACET_TYPES.add(FACET_FLOOD_DURATION_INFRASTRUCTURE_Q_LEFT);
        HANDLED_FACET_TYPES.add(FACET_FLOOD_DURATION_INFRASTRUCTURE_Q_RIGHT);
    }

    public static boolean isInfrastructureFacet(final String name) {
        return name.equals(FACET_FLOOD_DURATION_INFRASTRUCTURE_W_LEFT) || name.equals(FACET_FLOOD_DURATION_INFRASTRUCTURE_W_RIGHT)
                || name.equals(FACET_FLOOD_DURATION_INFRASTRUCTURE_Q_LEFT) || name.equals(FACET_FLOOD_DURATION_INFRASTRUCTURE_Q_RIGHT);
    }

    public static Facet createFloodDurationWCurveFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int facetIndex, final int resultIndex, final String description) {

        return new FloodDurationCurveFacet(FACET_FLOOD_DURATION_W, description);
    }

    public static Facet createFloodDurationQCurveFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int facetIndex, final int resultIndex, final String description) {

        return new FloodDurationCurveFacet(FACET_FLOOD_DURATION_Q, description);
    }

    public static Facet createMainValuesWFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int facetIndex, final int resultIndex, final String description) {

        return new FloodDurationMainValuesWFacet(FACET_FLOOD_DURATION_MAINVALUES_W, description);
    }

    public static Facet createMainValuesQFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int facetIndex, final int resultIndex, final String description, final Infrastructure infrastructure) {

        return new FloodDurationMainValuesQFacet(FACET_FLOOD_DURATION_MAINVALUES_Q, description, facetIndex, infrastructure);
    }

    public static Facet createInfrastructureFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int facetIndex, final int resultIndex, final boolean isW, final AttributeKey riverside) {

        final String description = getLabel(context, isW, riverside);
        final String facetName = getFacetName(riverside, isW);

        return new FloodDurationInfrastructureFacet(facetName, isW, resultIndex, riverside, facetIndex, description);
    }

    private static String getFacetName(final AttributeKey riverside, final boolean isW) {
        switch (riverside) {
        case LEFT:
            if (isW)
                return FACET_FLOOD_DURATION_INFRASTRUCTURE_W_LEFT;
            else
                return FACET_FLOOD_DURATION_INFRASTRUCTURE_Q_LEFT;

        case RIGHT:
            if (isW)
                return FACET_FLOOD_DURATION_INFRASTRUCTURE_W_RIGHT;
            else
                return FACET_FLOOD_DURATION_INFRASTRUCTURE_Q_RIGHT;
        default:
            throw new IllegalStateException();
        }
    }

    private static String getLabel(final CallContext context, final boolean isW, final AttributeKey riverside) {
        switch (riverside) {
        case LEFT:
            if (isW)
                return Resources.getMsg(context.getMeta(), "sinfo_facet_flood_duration_curve.infra.w.left.description");
            else
                return Resources.getMsg(context.getMeta(), "sinfo_facet_flood_duration_curve.infra.q.left.description");
        case RIGHT:
            if (isW)
                return Resources.getMsg(context.getMeta(), "sinfo_facet_flood_duration_curve.infra.w.right.description");
            else
                return Resources.getMsg(context.getMeta(), "sinfo_facet_flood_duration_curve.infra.q.right.description");

        default:
            throw new IllegalStateException();
        }
    }

    @Override
    public final String getAxisLabel(final DiagramGenerator generator) {
        return generator.msg(DurationCurveGenerator.I18N_YAXIS_LABEL_W);
    }

    @Override
    public final boolean canHandle(final String facettype) {
        return HANDLED_FACET_TYPES.contains(facettype);
    }
}